Attribute VB_Name = "MIDI_CMD"
' Dieser Source stammt von http://www.activevb.de
' und kann frei verwendet werden. Fr eventuelle Schden
' wird nicht gehaftet.
'
' Um Fehler oder Fragen zu klren, nutzen Sie bitte unser Forum.
' Ansonsten viel Spa und Erfolg mit diesem Source !
'
' Autor: K. Langbein Klaus@ActiveVB.de
'
' Chronologie:
' Dieser Sourcecode basiert auf der Vorarbeit von M. Levoi,
' mlevoi@modemss.brisnet.org.au. Die ursprnglichen Midi-Routinen wurden
' in MIDI_CMD.Bas zusammengefasst.
'
' MIDI_CMD.Bas History
' This Sample is based on the work of M. Levoi,
' http://www.modemss.brisnet.org.au/~mlevoi/index.html
' combined MIDI_CMD and MIDI_out to one file
' minor modifications (ie indentations changed)
' KL_midi relies on this module.
'

Dim m_hmidiout As Long

' **************************************************************************
'
'         Multimedia API Declares adapted from MMSYSTEM.H
'
'         Copyright (c) 1990-1993, Microsoft Corp.  All rights reserved.
'
' **************************************************************************

Global Const MIDIERR_BASE = 64

' ***************************************************************************

'                     General constants and data types

' ****************************************************************************/

'  general constants
Global Const MAXPNAMELEN = 32           '  max product name length (including NULL)
Global Const MAXERRORLENGTH = 128       '  max error text length (including NULL)


Global Const MM_MIM_OPEN = &H3C1                    '  MIDI input
Global Const MM_MIM_CLOSE = &H3C2
Global Const MM_MIM_DATA = &H3C3
Global Const MM_MIM_LONGDATA = &H3C4
Global Const MM_MIM_ERROR = &H3C5
Global Const MM_MIM_LONGERROR = &H3C6

Global Const MM_MOM_OPEN = &H3C7                    '  MIDI output
Global Const MM_MOM_CLOSE = &H3C8
Global Const MM_MOM_DONE = &H3C9

' ***************************************************************************

'                             MIDI audio support

' ****************************************************************************/

'  MIDI error return values
Global Const MIDIERR_UNPREPARED = (MIDIERR_BASE + 0)       '  header not prepared
Global Const MIDIERR_STILLPLAYING = (MIDIERR_BASE + 1)     '  still something playing
Global Const MIDIERR_NOMAP = (MIDIERR_BASE + 2)            '  no current map
Global Const MIDIERR_NOTREADY = (MIDIERR_BASE + 3)         '  hardware is still busy
Global Const MIDIERR_NODEVICE = (MIDIERR_BASE + 4)         '  port no longer connected
Global Const MIDIERR_INVALIDSETUP = (MIDIERR_BASE + 5)     '  invalid setup
Global Const MIDIERR_LASTERROR = (MIDIERR_BASE + 5)        '  last error in range

Global Const MIDIPATCHSIZE = 128

'  MIDI callback messages
Global Const MIM_OPEN = MM_MIM_OPEN
Global Const MIM_CLOSE = MM_MIM_CLOSE
Global Const MIM_DATA = MM_MIM_DATA
Global Const MIM_LONGDATA = MM_MIM_LONGDATA
Global Const MIM_ERROR = MM_MIM_ERROR
Global Const MIM_LONGERROR = MM_MIM_LONGERROR
Global Const MOM_OPEN = MM_MOM_OPEN
Global Const MOM_CLOSE = MM_MOM_CLOSE
Global Const MOM_DONE = MM_MOM_DONE

'  device ID for MIDI mapper
Global Const MIDIMAPPER = (-1)
Global Const MIDI_MAPPER = (-1)

'  flags for wFlags parm of midiOutCachePatches(), midiOutCacheDrumPatches()
Global Const MIDI_CACHE_ALL = 1
Global Const MIDI_CACHE_BESTFIT = 2
Global Const MIDI_CACHE_QUERY = 3
Global Const MIDI_UNCACHE = 4
'  flags for wTechnology field of MIDIOUTCAPS structure
Global Const MOD_MIDIPORT = 1      '  output port
Global Const MOD_SYNTH = 2         '  generic internal synth
Global Const MOD_SQSYNTH = 3       '  square wave internal synth
Global Const MOD_FMSYNTH = 4       '  FM internal synth
Global Const MOD_MAPPER = 5        '  MIDI mapper

'  flags for dwSupport field of MIDIOUTCAPS structure
Global Const MIDICAPS_VOLUME = &H1               '  supports volume control
Global Const MIDICAPS_LRVOLUME = &H2             '  separate left-right volume control
Global Const MIDICAPS_CACHE = &H4


'  flags for dwFlags field of MIDIHDR structure
Global Const MHDR_DONE = &H1                     '  done bit
Global Const MHDR_PREPARED = &H2                 '  set if header prepared
Global Const MHDR_INQUEUE = &H4                  '  reserved for driver

' MIDI output device capabilities structure
Type MIDIOUTCAPS
    wMid As Integer                ' Manufacturer ID
    wPid As Integer                ' Product ID
    vDriverVersion As Long         ' Driver version
    szPname As String * 32         ' Product name (NULL terminated string)
    wTechnology As Integer         ' Device type
    wVoices As Integer             ' n. of voices (internal synth only)
    wNotes As Integer              ' max n. of notes (internal synth only)
    wChannelMask As Integer        ' n. of Midi channels (internal synth only)
    dwSupport As Long              ' Supported extra controllers (volume, etc)
End Type

'
Private Declare Function midiOutGetNumDevs Lib "winmm" () As Integer

Private Declare Function midiOutGetDevCaps Lib "winmm.dll" _
                         Alias "midiOutGetDevCapsA" ( _
                         ByVal uDeviceID As Long, _
                         lpCaps As MIDIOUTCAPS, _
                         ByVal uSize As Long) As Long
                         
Private Declare Function midiOutGetVolume Lib "winmm.dll" ( _
                         ByVal uDeviceID As Long, _
                         lpdwVolume As Long) As Long
                         
Private Declare Function midiOutSetVolume Lib "winmm.dll" ( _
                         ByVal uDeviceID As Long, _
                         ByVal dwVolume As Long) As Long
                         
Private Declare Function midiOutGetErrorText Lib "winmm.dll" _
                         Alias "midiOutGetErrorTextA" ( _
                         ByVal err As Long, _
                         ByVal lpText As String, _
                         ByVal uSize As Long) As Long
                         
Private Declare Function MIDIOutOpen Lib "winmm.dll" _
                         Alias "midiOutOpen" ( _
                         lphMidiOut As Long, _
                         ByVal uDeviceID As Long, _
                         ByVal dwCallback As Long, _
                         ByVal dwInstance As Long, _
                         ByVal dwFlags As Long) As Long
                         
Public Declare Function midiOutClose Lib "winmm.dll" ( _
                        ByVal hMidiOut As Long) As Long
                        
Private Declare Function midiOutShortMsg Lib "winmm.dll" ( _
                         ByVal hMidiOut As Long, _
                         ByVal dwMsg As Long) As Long

Sub midi_listoutdevs(c As Control)

    ' must receive a list or combo control to take the list

    Dim i As Integer
    Dim x As Integer
    Dim midicaps As MIDIOUTCAPS

    c.Clear
    c.AddItem "No selection"
    ' Test for MIDI mapper
    If midiOutGetDevCaps(MIDIMAPPER, midicaps, Len(midicaps)) = 0 Then ' OK
        c.AddItem midicaps.szPname
        c.ItemData(c.NewIndex) = MIDIMAPPER ' Save dev_id in item data
    End If
    
    ' Add other devs
    For i = 0 To midiOutGetNumDevs() - 1
        If midiOutGetDevCaps(i, midicaps, Len(midicaps)) = 0 Then ' OK
            c.AddItem midicaps.szPname
            c.ItemData(c.NewIndex) = i ' Save dev_id
        End If
    Next
    
End Sub
Sub midi_out_close()
    
    Dim midi_error As Integer

    If m_hmidiout <> 0 Then
        midi_error = midiOutClose(m_hmidiout)
        If Not midi_error = 0 Then
            Call midi_outerr(midi_error)
        End If
        m_hmidiout = 0
    End If
    
End Sub

Function midi_out_open(ByVal dev_id As Integer) As Integer

    Dim midi_error As Integer

    midi_out_close ' just in case (And it dont hurt)
    midi_error = MIDIOutOpen(m_hmidiout, dev_id, 0, 0, 0)
    If Not midi_error = 0 Then
        Call midi_outerr(midi_error)
    End If
    If m_hmidiout <> 0 Then
        midi_out_open = -1
        x = write_handle(chop_slash(App.Path) + "\handles.log", "Midi-handle", m_hmidiout)
    Else
       
    End If
    
End Function

Sub midi_outerr(ByVal midi_error As Integer)

    Dim s As String
    Dim x As Integer

    s = Space(MAXERRORLENGTH)
    x = midiOutGetErrorText(midi_error, s, MAXERRORLENGTH)
    'If Not g_debug Then
        MsgBox s
    'End If
    
End Sub

Sub midi_outshort(b1 As Integer, b2 As Integer, b3 As Integer)

    Dim midi_error As Integer
    
    If m_hmidiout = 0 Then
        Exit Sub
    End If

    midi_error = midiOutShortMsg(m_hmidiout, packdword(0, b3, b2, b1))
    If Not midi_error = 0 Then
        Call midi_outerr(midi_error)
    End If
    
End Sub



Function packdword(i1 As Integer, i2 As Integer, i3 As Integer, i4 As Integer) As Long

    On Error Resume Next
    packdword = i2 * &H10000 + i3 * &H100 + i4
    
End Function


Sub all_sounds_off()

    Dim channel As Integer

    For channel = 0 To 15
        Call midi_outshort(&HB0 + channel, &H78, 0)
        Call midi_outshort(&HB0 + channel, &H7B, 0)
    Next
    
End Sub


Sub control_change(ch As Integer, ccnr As Integer, ByVal v As Integer)

    Call midi_outshort(&HB0 + ch, ccnr, v)
    
End Sub

Sub note_off(ch As Integer, ByVal kk As Integer)

    Call midi_outshort(&H90 + ch, kk, 0)
    
End Sub

Sub note_on(ch As Integer, ByVal kk As Integer, v As Integer)

    ch = ch + 0
    Call midi_outshort(&H90 + ch, kk, v)
    
End Sub


Sub program_change(ch As Integer, cc0nr As Integer, ByVal pnr As Integer)

    Call midi_outshort(&HB0 + ch, cc0nr, 0)
    Call midi_outshort(&HC0 + ch, pnr, 0)
    
End Sub

